'''
@copyright: 2022 - Symas Corporation
'''

import unittest
from rbac.ldap import userdao, permdao, roledao, InvalidCredentials
from rbac.model import User, Perm, Role, PermObj
from rbac.tests import user_test_data, role_test_data
from rbac.cli.utils import print_user, print_role, print_ln, print_entity
from rbac.tests import perm_test_data


class BasicTestSuite(unittest.TestCase):
    """These tests assume fortress user and permission data has been pre-loaded into Ldap, i.e. via apache fortress administrative functions."""
                
class TestDaos(unittest.TestCase):
    """
    Test the user functions from the user and perm dao modules
    """    
    
    def test_search_users(self):
        """
        Test the user search by uid in ldap
        """
        print_ln('test search users by uid')        
        try:
            usr = User(uid = "jts*")
            uList = userdao.search(usr)
            for idx, entity in enumerate(uList) :            
                print_user(entity, "User[" + str(idx+1) + "]:")
        except Exception as e:
            self.fail('user search failed, exception=' + e.msg)

            
    def test_search_perms(self):
        """
        Test the perm search by obj_name and op_name in ldap
        """
        print_ln('test search perms by objNm')        
        try:
            prm = Perm(obj_name = "TOB*", op_name = "TOP*")
            pList = permdao.search(prm)
            for idx, entity in enumerate(pList) :            
                print_entity (entity, "Perm[" + str(idx+1) + "]:", 1)
        except Exception as e:
            self.fail('perm search failed, exception=' + e.msg)

            
    def test_bind_users(self):
        """
        Test the user bind
        """
        print_ln('test bind users')        
        try:
            usr = User(uid = "jtsuser*")
            uList = userdao.search(usr)
            for idx, entity in enumerate(uList) :
                entity.password = 'passw0rd' + str(idx+1)                
                try:      
                    userdao.authenticate(entity)
                except InvalidCredentials as e:
                    print_ln(e.msg)
                    #self.fail('user bind invalid creds, user=' + entity.uid)
                          
        except Exception as e:
            self.fail('user bind exception=' + e.msg)


    def test_bind_users_negative(self):
        """
        Test the user bind
        """
        print_ln('test bind users')        
        try:
            usr = User(uid = "jtsuser*")
            uList = userdao.search(usr)
            for idx, entity in enumerate(uList) :
                entity.password = 'notrightpassword'
                try:      
                    userdao.authenticate(entity)
                    self.fail('test bind negative failed ' + entity.uid)
                except InvalidCredentials:
                    pass
                         
        except Exception as e:
            self.fail('user bind failed, exception=' + e.msg)


    def test_search_roles(self):
        """
        Test the role search by name in ldap
        """
        print_ln('test search roles by name')        
        try:
            rle = Role(name = 'oam*')
            rList = roledao.search(rle)
            for idx, entity in enumerate(rList) :            
                print_role(entity, "Role[" + str(idx+1) + "]:")
        except Exception as e:
            self.fail('role search failed, exception=' + e.msg)

                        
    def test_create_roles(self):
        """
        Test the role create
        """
        print_ln('test create roles')
        rls = role_test_data.get_test_roles('py-test', 10)
        for rle in rls:
            try:                        
                rle = roledao.create(rle)
                print_role(rle, "Role Create")
            except Exception as e:
                self.fail('role create failed, exception=' + e.msg)


    def test_update_roles(self):
        """
        Test the role update
        """
        print_ln('test update roles')
        rls = role_test_data.get_test_roles('py-test', 10)
        for rle in rls:
            rle.description += '-updated'
            try:                        
                rle = roledao.update(rle)
                print_role(rle, "Role Update")
            except Exception as e:
                self.fail('role update failed, exception=' + e.msg)


    def test_delete_roles(self):
        """
        Test the role delete
        """
        print_ln('test delete roles')
        
        try:
            rList = roledao.search(Role(name='py-test*'))
            for rle in rList:                       
                rle = roledao.delete(rle)
                print_ln("Role Delete role=" + rle.name)
        except Exception as e:
            self.fail('role delete failed, exception=' + e.msg)


    def test_create_users(self):
        """
        Test the user create
        """
        print_ln('test create users')
        usrs = user_test_data.get_test_users('py-test', 10)
        for usr in usrs:
            try:                        
                entity = userdao.create(usr)
                print_user(entity, "User Create")
            except Exception as e:
                self.fail('user create failed, exception=' + e.msg)


    def test_update_users(self):
        """
        Test the user update
        """
        print_ln('test update users')
        usrs = user_test_data.get_test_users('py-test', 10)
        for usr in usrs:
            usr.description += '-updated'
            usr.cn += '-updated'
            usr.sn += '-updated'                        
            try:                        
                entity = userdao.update(usr)
                print_user(entity, "User Update")
            except Exception as e:
                self.fail('user update failed, exception=' + e.msg)


    def test_delete_users(self):
        """
        Test the user delete
        """
        print_ln('test delete users')
        
        try:
            uList = userdao.search(User(uid='py-test*'))
            for usr in uList:                       
                entity = userdao.delete(usr)
                print_ln("Delete user=" + entity.uid)
        except Exception as e:
            self.fail('user delete failed, exception=' + e.msg)


    def test_create_objects(self):
        """
        Test the object create
        """
        print_ln('test create objects')
        objs = perm_test_data.get_test_objs('py-test', 10)
        for obj in objs:
            try:                        
                entity = permdao.create_obj(obj)
                print_ln("Create object=" + entity.obj_name)                
            except Exception as e:
                self.fail('perm object create failed, exception=' + e.msg)


    def test_update_objects(self):
        """
        Test the object update
        """
        print_ln('test update objects')
        objs = perm_test_data.get_test_objs('py-test', 10)
        for obj in objs:
            obj.description += '-updated'
            obj.type += '-updated'                        
            try:                        
                entity = permdao.update_obj(obj)
                print_ln("Update object=" + entity.obj_name)                
            except Exception as e:
                self.fail('object update failed, exception=' + e.msg)


    def test_delete_objects(self):
        """
        Test the object delete
        """
        print_ln('test delete objects')
        
        try:
            oList = permdao.search_objs(PermObj(obj_name='py-test*'))
            for obj in oList:                       
                entity = permdao.delete_obj(obj)
                print_ln("Delete object=" + obj.obj_name)
        except Exception as e:
            self.fail('perm obj delete failed, exception=' + e.msg)


    def test_create_perms(self):
        """
        Test the perm create
        """
        print_ln('test create perms')
        perms = perm_test_data.get_test_perms('py-test', 10)
        for perm in perms:
            try:                        
                entity = permdao.create(perm)
                print_ln("Create perm obj=" + entity.obj_name + ', op=' + entity.op_name + ', id=' + entity.obj_id)                
            except Exception as e:
                self.fail('perm create failed, exception=' + e.msg)


    def test_update_perms(self):
        """
        Test the perm update
        """
        print_ln('test update perms')
        perms = perm_test_data.get_test_perms('py-test', 10)
        for perm in perms:
            perm.description += '-updated'
            perm.type += '-updated'                        
            try:                        
                entity = permdao.update(perm)
                print_ln("Update perm obj=" + entity.obj_name + ', op=' + entity.op_name + ', id=' + entity.obj_id)                
            except Exception as e:
                self.fail('perm update failed, exception=' + e.msg)


    def test_delete_perms(self):
        """
        Test the perm delete
        """
        print_ln('test delete perms')
        
        try:
            pList = permdao.search(Perm(obj_name='py-test*', op_name='*'))
            for perm in pList:                       
                entity = permdao.delete(perm)
                print_ln("Delete perm obj=" + perm.obj_name + ', op=' + perm.op_name + ', id=' + perm.obj_id)
        except Exception as e:
            self.fail('perm delete failed, exception=' + e.msg)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(TestDaos('test_search_users'))
    suite.addTest(TestDaos('test_bind_users'))
    suite.addTest(TestDaos('test_bind_users_negative'))               
    suite.addTest(TestDaos('test_search_perms'))   
    suite.addTest(TestDaos('test_search_roles'))
    suite.addTest(TestDaos('test_delete_roles'))    
    suite.addTest(TestDaos('test_create_roles'))
    suite.addTest(TestDaos('test_update_roles'))
    suite.addTest(TestDaos('test_delete_users'))
    suite.addTest(TestDaos('test_create_users'))
    suite.addTest(TestDaos('test_update_users'))
    suite.addTest(TestDaos('test_delete_perms'))    
    suite.addTest(TestDaos('test_delete_objects'))
    suite.addTest(TestDaos('test_create_objects'))             
    suite.addTest(TestDaos('test_update_objects'))
    suite.addTest(TestDaos('test_create_perms'))
    suite.addTest(TestDaos('test_update_perms'))            
    return suite  

 
if __name__ == '__main__':
    runner = unittest.TextTestRunner(failfast=True)
    runner.run(suite())


'''
@copyright: 2022 - Symas Corporation
'''

from rbac.util.fortress_error import RbacError

class LdapException(RbacError):
 pass

class NotFound(RbacError):
 pass

class NotUnique(RbacError):
 pass

class InvalidCredentials(RbacError):
 pass



'''
@copyright: 2022 - Symas Corporation
'''
from setuptools import setup
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='py-fortress',
      version='0.1.7',
      python_requires='>=3.6',
      description='RBAC for Python 3',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/shawnmckinney/py-fortress',
      author='Symas Corporation',
      author_email='smckinney@symas.com',
      license='Apache License 2.0',
      packages=['rbac', 'rbac.cli', 'rbac.ldap', 'rbac.model', 'rbac.util', 'rbac.tests', 'doc'],
      package_data={
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.md'],
      },
      install_requires=[
          'python-ldap >= 3.4.2',
          'six >= 1.16.0',
          'ldappool >= 3.0.0'
      ],
      classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',  
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Security',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',          
      ],
      keywords='authorization rbac security', 
      entry_points={
       'console_scripts': [
           'initldap = rbac.tests.test_dit_dao:main',
           'cli = rbac.cli.cli:main',
           'clitest = rbac.cli.cli_test_auth:main',
        ],
      },
      data_files=[('conf', ['py-fortress-cfg.json.sample'])],
      zip_safe=False)



'''
@copyright: 2012 - Symas Corporation
'''

from rbac.model import PermObj, Perm
from enum import Enum


def get_test_objs(name, total):
    objs = []
    for i in range(total):
        objs.append(__create_test_obj( name, i))
    return objs

    
def __create_test_obj(name, number):    
    label = name + '-' + str(number)
    return PermObj(
        obj_name=label, 
        description=label + ' Object',
        ou='py-test',
        type='test',
        props = ['name1:value1', 'name2:value2']        
    )
    
    
def get_test_perms(name, total):
    perms = []
    for i in range(total):
        for op in Operations:             
            perms.append(__create_test_perm( name, i, op.name))
    return perms

    
def __create_test_perm(name, number, op):    
    label = name + '-' + str(number)
    return Perm(
        obj_name=label,
        op_name=op, 
        obj_id=str(number),
        description=label + ' Object.' + op,
        abstract_name=label + '.' + op,
        type='test',
        props = ['name1:value1', 'name2:value2']        
    )    
    
    
class Operations(Enum):
    ADD = 1
    UPDATE = 2
    DELETE = 3
    READ = 4
    SEARCH = 5
    LOAD = 6
    MONITOR = 7
    NOTIFY = 8
    TERMINATE = 9
    INITIATE = 10
        


'''
@copyright: 2022 - Symas Corporation
'''

from . import Validator
from .global_ids import ACTV_FAILED_TIMEOUT, SUCCESS

class TimeOut(Validator):
    
    def validate(self, constraint, now, session):
        # this is the expected condition for a new session i.e. first ever access:
        if session.last_access is None:
            return SUCCESS
        rc = ACTV_FAILED_TIMEOUT
        if constraint.timeout == 0:
            rc = SUCCESS
        else:
            elapsed_time = now.seconds - session.last_access.seconds;
            time_limit = constraint.timeout * 60
            if elapsed_time < time_limit  or constraint.timeout == 0:
                rc = SUCCESS
        return rc       


'''
@copyright: 2022 - Symas Corporation
'''

import unittest
from rbac import admin, review
from rbac.model import User, Role, Perm, PermObj
from rbac.cli.utils import print_user, print_role, print_ln
from rbac.tests import user_test_data, role_test_data
from rbac.tests import perm_test_data


class BasicTestSuite(unittest.TestCase):
    """These tests the py-fortress admin functions."""
                

class TestAdminMgr(unittest.TestCase):
    """
    Test the admin funcs
    """    

    def test10_add_user(self):
        """
        Test the add user method
        """
        print_ln('test_add_user')
        usrs = user_test_data.get_test_users('py-user', 10)
        for usr in usrs:
            try:                        
                entity = admin.add_user(usr)
                print_user(entity, "Add User")
            except Exception as e:
                self.fail('test_add_user failed, exception=' + e.msg)


    def test05_delete_user(self):
        """
        Test the user delete user method
        """
        print_ln('test_delete_user')
        
        try:
            # TODO: search for roles assigned to user and remove the occupant
            uList = review.find_users(User(uid='py-user*'))
            for usr in uList:                       
                entity = admin.delete_user(usr)
                print_ln("Delete user=" + entity.uid)
        except Exception as e:
            self.fail('test_delete_user failed, exception=' + e.msg)


    def test07_add_role(self):
        """
        Test the add role method
        """
        print_ln('test_add_role')
        rles = role_test_data.get_test_roles('py-role', 10)
        for rle in rles:
            try:                        
                entity = admin.add_role(rle)
                print_role(entity, "Add Role")
            except Exception as e:
                self.fail('test_add_role failed, exception=' + e.msg)


    def test06_delete_role(self):
        """
        Test the role delete user method
        """
        print_ln('test_delete_role')
        
        try:
            rList = review.find_roles(Role(name='py-role*'))
            for rle in rList:                       
                entity = admin.delete_role(rle)
                print_ln("Delete role=" + entity.name)
        except Exception as e:
            self.fail('test_delete_role failed, exception=' + e.msg)


    def test08_add_object(self):
        """
        Test the add object method
        """
        print_ln('test_add_object')
        objs = perm_test_data.get_test_objs('py-obj', 10)
        for obj in objs:
            try:                        
                entity = admin.add_object(obj)
                print_ln("Add Object=" + entity.obj_name)                
            except Exception as e:
                self.fail('test_add_object failed, exception=' + e.msg)


    def test03_delete_object(self):
        """
        Test the role delete object method
        """
        print_ln('test_delete_object')
        
        try:
            oList = review.find_objects(PermObj(obj_name='py-obj*'))
            for obj in oList:                       
                admin.delete_object(obj)
                print_ln("Delete Object=" + obj.obj_name)
        except Exception as e:
            self.fail('test_delete_object failed, exception=' + e.msg)


    def test09_add_perm(self):
        """
        Test the add perm method
        """
        print_ln('test_add_perm')
        perms = perm_test_data.get_test_perms('py-obj', 10)
        for perm in perms:
            try:                        
                entity = admin.add_perm(perm)
                print_ln("Add Perm obj name=" + entity.obj_name + ', op=' + entity.op_name + ', id=' + entity.obj_id)                                
            except Exception as e:
                self.fail('test_add_perm failed, exception=' + e.msg)


    def test02_delete_perm(self):
        """
        Test the perm delete object method
        """
        print_ln('test_delete_perm')
        
        try:
            pList = review.find_perms(Perm(obj_name='py-obj*', op_name='*'))
            for perm in pList:                       
                entity = admin.delete_perm(perm)
                print_ln("Delete Perm obj name=" + entity.obj_name + ', op=' + entity.op_name + ', id=' + entity.obj_id)
        except Exception as e:
            self.fail('test_delete_perm failed, exception=' + e.msg)


    def test11_assign_user(self):
        """
        Test the assign user method
        """
        print_ln('test_assign_user')
        usrs = user_test_data.get_test_users('py-user', 10)
        rles = role_test_data.get_test_roles('py-role', 10)        
        for usr in usrs:
            for rle in rles:
                try:                        
                    admin.assign(usr, rle)
                    print_ln("Assign User=" + usr.uid + ', Role=' + rle.name)
                except Exception as e:
                    self.fail('test_assign_user failed, exception=' + e.msg)


    def test04_deassign_user(self):
        """
        Test the user deassign method
        """
        print_ln('test_deassign_user')
        
        try:
            uList = review.find_users(User(uid='py-user*'))
            rles = role_test_data.get_test_roles('py-role', 10)                
            for usr in uList:
                for rle in rles:                                       
                    entity = admin.deassign(usr, rle)
                    print_ln("Deassign User=" + entity.uid + ', Role=' + rle.name)
        except Exception as e:
            pass
            #self.fail('test_deassign_user failed, exception=' + e.msg)


    def test12_grant(self):
        """
        Test the grant method
        """
        print_ln('test_grant')
        perms = perm_test_data.get_test_perms('py-obj', 10)
        rles = role_test_data.get_test_roles('py-role', 10)        
        for perm in perms:
            for rle in rles:
                try:                        
                    admin.grant(perm, rle)
                    print_ln("Grant Perm obj name=" + perm.obj_name + ', op=' + perm.op_name + ', id=' + perm.obj_id + ', Role=' + rle.name)                                
                except Exception as e:
                    self.fail('test_grant failed, exception=' + e.msg)


    def test01_revoke(self):
        """
        Test the revoke method
        """
        print_ln('test_revoke')
        
        try:
            pList = review.find_perms(Perm(obj_name='py-obj*', op_name='*'))
            rles = role_test_data.get_test_roles('py-role', 10)                                    
            for perm in pList:                       
                for rle in rles:
                    admin.revoke(perm, rle)
                    print_ln("Revoke Perm obj name=" + perm.obj_name + ', op=' + perm.op_name + ', id=' + perm.obj_id + ', Role=' + rle.name)                                
        except Exception as e:
            pass
            #self.fail('test_revoke failed, exception=' + e.msg)


def suite():
    suite = unittest.TestSuite()
    
    # Teardown:
    suite.addTest(TestAdminMgr('test01_revoke'))
    suite.addTest(TestAdminMgr('test02_delete_perm'))
    suite.addTest(TestAdminMgr('test03_delete_object'))
    suite.addTest(TestAdminMgr('test04_deassign_user'))
    suite.addTest(TestAdminMgr('test05_delete_user'))
    suite.addTest(TestAdminMgr('test06_delete_role'))
    
    # Buildup:
    suite.addTest(TestAdminMgr('test07_add_role'))
    suite.addTest(TestAdminMgr('test08_add_object'))
    suite.addTest(TestAdminMgr('test09_add_perm'))
    suite.addTest(TestAdminMgr('test10_add_user'))
    suite.addTest(TestAdminMgr('test11_assign_user'))
    suite.addTest(TestAdminMgr('test12_grant'))
    return suite  

 
if __name__ == '__main__':
    runner = unittest.TextTestRunner(failfast=True)
    runner.run(suite())


'''
@copyright: 2022 - Symas Corporation
'''

import unittest
from rbac.util import Date
from rbac.util import Day
from rbac.util import LockDate
from rbac.util import Time
from rbac.util import TimeOut
from rbac.util import CurrentDateTime
from rbac.model import Constraint
from rbac.cli.utils import print_ln, print_entity

cons1 = [
    Constraint(
        name='foo1',
        timeout='0',
        begin_time='0100',
        end_time='2400',
        begin_date='20170101',
        end_date='20171231',
        begin_lock_date='20170101',
        end_lock_date='20171231',
        day_mask='1234567',
        ),
    Constraint(
        name='foo2',
        timeout='0',
        begin_time='0100',
        end_time='2400',
        begin_date='20170101',
        end_date='20171231',
        begin_lock_date='20180101',
        end_lock_date='20181231',
        day_mask='1234567',
        ),
    Constraint(
        name='foo3',
        timeout='0',
        begin_time='0100',
        end_time='2400',
        begin_date='20170101',
        end_date='20181231',
        begin_lock_date='20170101',
        end_lock_date='20171231',
        day_mask='1234567',
        ),
    ]

class BasicTestSuite(unittest.TestCase):
    """Verify the temporal validators against the current system date with some sample constraint values."""
                
class TestConstraints(unittest.TestCase):
    
        
    def test_validators(self):
        """
        Test the temporal constraints
        """
        print('test time-based constraints')
        validators = []
        validators.append(Date())
        validators.append(Day())
        validators.append(LockDate())
        validators.append(Time())
        validators.append(TimeOut())
        for constraint in cons1:
            for validator in validators:
                result = validator.validate(constraint, CurrentDateTime())
                print_entity (constraint, "Validate" + str(validator))
                print_ln( 'result=' + str(result), 1 )
    

def suite():
    suite = unittest.TestSuite()
    suite.addTest(TestConstraints('test_validators'))
    return suite  


if __name__ == '__main__':
    runner = unittest.TextTestRunner(failfast=True)
    runner.run(suite())


if __name__ == "__main__":
    unittest.main()